#define Num_Fre		4
#define Max_Fre		3


#define TwInqScan_c	4



/* the interval between two consecutive inquiry scans.
The inquiry scan interval shall be at most 4096 time slots.
*/
#define TinqScan_c      7	/* IDSent*TrainSent*2 */




/* 2048 time slots */
#define PhaseChange_c	3	/* IDSent*TrainSent */

/* tarin offset */
#define TRAIN_A         1	/* Num_Fre/4 */
#define TRAIN_B         3	/* Num_Fre*(3/4) */



/* used in INQUIRY process */
#define IDSent          1	/* Num_Fre/2 */
#define TrainSent       1       /* 2 tarins (0,1) */
#define TrainSwitch	3



/* state definition */
#define INQUIRY_SCAN	0
#define INQUIRY		1
#define CONNECTED	2


/*************************************
	variable definition
**************************************/
process count = 2;

/* inquiry scan */
global discrete fre_1: 0...3;
global discrete fre_base_1: 0...3;
global discrete state_1: 0...2;
global discrete phase_clk_1: 0...3;	/* used to update fre_base_1 */
global discrete inqscanTimer_: 0...7;	/* used to change state between 'scan' and 'connect'*/
global clock tick_clk_1;		/* used to update fre_2 */


/* inquiry */
global discrete fre_2: 0...7;
global discrete fre_base_2: 0...3;
global discrete phase_clk_2: 0...3;	/* used to update fre_base_2 */
global clock tick_clk_2;		/* used to update fre_2 */

global discrete offset: 0..3;

global discrete id_sent: 0...2;
global discrete train_sent: 0...1;
global discrete train_switch: 0...3;
global discrete clkmod: 0...3;


global synchronizer signal_packet, signal_success;


/* scan */
mode  update_fre_base_1 tick_clk_1<=1{
        /* xtion 0 */
	when tick_clk_1==1 and phase_clk_1!=PhaseChange_c
	may tick_clk_1=0; phase_clk_1++1; goto update_state_1;

        /* xtion 1 */
        when tick_clk_1==1 and phase_clk_1==PhaseChange_c and fre_base_1<Max_Fre
        may tick_clk_1=0; phase_clk_1=0; fre_base_1++1; goto update_state_1;

        /* xtion 2 */
        when tick_clk_1==1 and phase_clk_1==PhaseChange_c and fre_base_1==Max_Fre
        may tick_clk_1=0; phase_clk_1=0; fre_base_1=0; goto update_state_1;


}

mode  update_state_1 tick_clk_1==0{

	/* xtion 3 */
	when inqscanTimer_<TwInqScan_c may inqscanTimer_++1; goto update_fre_base_1;

    	/* xtion 4 */
	when inqscanTimer_==TwInqScan_c may inqscanTimer_++1; state_1=CONNECTED; goto update_fre_base_1;

	/* xtion 5 */
	when  TwInqScan_c<inqscanTimer_ and inqscanTimer_<TinqScan_c may inqscanTimer_++1; goto update_fre_base_1;

    	/* xtion 6 */
	when inqscanTimer_==TinqScan_c may inqscanTimer_=0; state_1=INQUIRY_SCAN; fre_1=fre_base_1; goto update_fre_base_1;

}






/* inquiry */

/* update_fre_base_2 to next time slot */

mode update_fre_base_2 tick_clk_2<=1{
        /* xtion 7 */
	when phase_clk_2!=PhaseChange_c
	may phase_clk_2++1;
        goto update_clkmod;

        /* xtion 8 */
        when phase_clk_2==PhaseChange_c and fre_base_2<Max_Fre
        may phase_clk_2=0; fre_base_2++1;
        goto update_clkmod;

        /* xtion 9 */
        when phase_clk_2==PhaseChange_c and fre_base_2==Max_Fre
        may phase_clk_2=0; fre_base_2=0;
        goto update_clkmod;
}

mode update_clkmod tick_clk_2<=1 {
        /* xtion 10 */
        when tick_clk_2==0 and clkmod==0 may clkmod=1; fre_2=fre_base_2+offset+id_sent; goto mod;

        /* xtion 11 */
        when tick_clk_2==0 and clkmod==1 may clkmod=2; fre_2=fre_base_2+offset+id_sent; goto mod;

        /* xtion 12 */
        when tick_clk_2==1 and clkmod==2 may tick_clk_2=0; clkmod=3; goto update_fre_base_2;

        /* xtion 13 */
        when tick_clk_2==1 and clkmod==3 may tick_clk_2=0; clkmod=0; goto update_fre_base_2;

}

/* mod and check
   clkmode4=1 */
mode mod tick_clk_2<=1 {
        /* xtion 14 */
        when fre_2<=Max_Fre
        may goto check;

        /* xtion 15 */
        when fre_2>Max_Fre
        may fre_2=fre_base_2+offset+id_sent-4; goto check;

}


mode check tick_clk_2<=1 {
        when !signal_packet true may;
}


/* check timeout */
mode s2 tick_clk_2<=1 {
	/* id sent */
        /* xtion 19 */
	when tick_clk_2==1 and id_sent<IDSent
        may tick_clk_2=0; id_sent++1;
        goto update_fre_base_2;

	/* train repeat*/
        /* xtion 20 */
	when tick_clk_2==1 and id_sent==IDSent and train_sent<TrainSent
        may tick_clk_2=0; id_sent=0; train_sent++1;
	goto update_fre_base_2;

	/* train switch*/
        /* xtion 21 */
	when tick_clk_2==1 and id_sent==IDSent and train_sent==TrainSent and train_switch<TrainSwitch and offset==TRAIN_A
        may tick_clk_2=0; id_sent=0; train_sent=0; train_switch++1; offset=TRAIN_B;
	goto update_fre_base_2;

        /* xtion 22 */
	when tick_clk_2==1 and id_sent==IDSent and train_sent==TrainSent and train_switch<TrainSwitch and offset==TRAIN_B
        may tick_clk_2=0; id_sent=0; train_sent=0; train_switch++1; offset=TRAIN_A;
	goto update_fre_base_2;

        /* tiimeout */
        /* xtion 23 */
	when id_sent==IDSent and train_sent==TrainSent and train_switch==TrainSwitch
        may goto timeout;

}


mode success true {
}


mode timeout true {
}

initially

        update_fre_base_1[1] and

        ((inqscanTimer_<TwInqScan_c and state_1==INQUIRY_SCAN) or
         (inqscanTimer_>=TwInqScan_c and state_1==CONNECTED))

        and
        ((inqscanTimer_<4 and phase_clk_1==inqscanTimer_ and fre_1==fre_base_1) or
         (inqscanTimer_>=4 and phase_clk_1==inqscanTimer_-4 and fre_1==fre_base_1-1))

        and

        update_clkmod[2] and tick_clk_2==0 and offset==TRAIN_A and id_sent==0 and train_sent==0 and train_switch==0
        and phase_clk_2==0 and clkmod==0
        ;

risk
        timeout[2];


