/* Knuth elevator with 2 floor */
/*Every 50 unit time, there will be someone into the system*/

#define INTERVALTIME 30
#define GIVEUPTIME 100
#define OPENTIME 20
#define MOVETIME 20
#define CLOSETIME 40
#define SYSTEMLIMIT 8
#define QUEUELIMIT 4
#define ELEVATORLIMIT 6


process count = 5;

global pointer floor;

global discrete system:0..9; /*Count the number of current people in system*/

local clock x;
local discrete queueLength:0..5;/*the waiting queue for each floor*/
local discrete elevatorLength:0..7;/*the servinig queue in elevator*/
local discrete call:0..1; /*for each floor, it records that someone push the call bottom while the elevator doesn't in thie floor*/

global synchronizer PEOPLE_IN, PEOPLE_OUT, ELEVATOR_OPENREQUEST, PEOPLE_SERVED, ELEVATOR_IN, DOOR_OPEN, DOOR_CLOSED, CALL;

/*Generate people move into the system */
mode add x<=INTERVALTIME {
	when !PEOPLE_IN (x==INTERVALTIME and system<=SYSTEMLIMIT) may x=0; system=system+1;
	when x==INTERVALTIME and system > SYSTEMLIMIT may x=0;
}

/*People Served, GiveUp*/
mode withdraw x<=GIVEUPTIME {
	when !PEOPLE_OUT x==GIVEUPTIME and system>0 may x=0; system=system-1;
	when ?PEOPLE_SERVED true may x=0;
}

/*Floor queue*/
mode wait true{
	when !CALL floor!=P and queueLength>0 may call=1; /*people put botton*/
	when ?PEOPLE_IN floor!=P and queueLength<=QUEUELIMIT may queueLength=queueLength+1; /*someone get in*/
	when ?PEOPLE_OUT queueLength>0 may queueLength=queueLength-1;/*someone give up*/
	when ?ELEVATOR_IN call==1 may call=0; floor=P; goto  elevatorin;
	when ?PEOPLE_IN !ELEVATOR_OPENREQUEST floor==P and queueLength<=QUEUELIMIT may queueLength=queueLength+1; goto  elevatorin;
	when !ELEVATOR_OPENREQUEST floor==P and queueLength>0 may goto  elevatorin;
}

mode elevatorin true{
	when ?DOOR_OPEN !PEOPLE_SERVED floor==P may goto elevatorout;
	when ?PEOPLE_IN floor==P and queueLength<=QUEUELIMIT may queueLength=queueLength+1; /*floor==P and queueLength<QUEUELIMIT?*/
}

mode elevatorout true{
	when ?DOOR_CLOSED floor==P may queueLength=0; goto wait;
}

/*Elevator*/
mode neutral true{
	when ?ELEVATOR_OPENREQUEST true may x=0;goto opendoor;
	when ?CALL true may x=0; goto move;
}

mode closedoor x<=CLOSETIME{
	when ?ELEVATOR_OPENREQUEST true may x=0; goto opendoor;
	when !DOOR_CLOSED x==CLOSETIME may x=0; goto neutral;
	when ?PEOPLE_IN !PEOPLE_SERVED elevatorLength<ELEVATORLIMIT may x=0; elevatorLength=elevatorLength+1; /*allow someone get into the elevator without queueing*/
}

mode opendoor x<=OPENTIME{
	when ?ELEVATOR_OPENREQUEST true may ;
	when !DOOR_OPEN x==OPENTIME and system>=elevatorLength may system=system-elevatorLength; x=0; goto peopleinto;
	when !DOOR_OPEN x==OPENTIME and system<elevatorLength may system=0; x=0; goto peopleinto; /*an error transition*/
}

mode peopleinto x<=0 {
	when x==0 may elevatorLength=floor->queueLength; x=0; goto closedoor;
}

mode move x<=MOVETIME{
	when !ELEVATOR_IN x==MOVETIME may x=0; goto opendoor;
	when x==MOVETIME may x=0; goto neutral;

}




initially
	floor==4 and system==0 and
	add[1] and withdraw[2] and
	neutral[3] and elevatorLength[3]==0 and
	forall i, (x[i]==0) and
	forall p:p>3, (wait[p] and call[p] == 0 and queueLength[p]==0);

/* liveness */

specification
forall always forall p:p>3, (call[p]==1 implies forall eventually floor==p);
