/* -- Generalized Railway Crossing example
-- synthesize for the safety, when trains in  intersection => gate down
*/

process count = 5;

local dense 	x; /* clocks for train 1 and train 2 and gate angle for gate */
parameter	CUTOFF;
global synchronizer
	enterR1, exit1,
	enterR2, exit2,
	enterR3, exit3,
	lower, raise;

/*0*/
mode g_open true {
  rate x:0;
  when ?lower true may goto g_lowering;
  when ?raise true may ;
}
/*1*/
mode g_closed true {
  rate x:0;
  when ?lower true may goto g_closed;
  when ?raise true may goto g_raising;
}
/*2*/
mode g_raising x<=90 {
  rate x: [9,10];
  when ?raise true may goto g_raising;
  when ?lower true may goto g_lowering;
  when x==90 may goto g_open;
}
/*3*/
mode g_lowering x>=0 {
  rate x:[-10,-9];
  when ?raise true may goto g_raising;
  when ?lower true may goto g_lowering;
  when x==0 may goto g_closed;
}


/*4*/
mode up forall i:i>2, (x[i]<=CUTOFF) {
  when ?exit1 true may ;
  when ?exit2 true may ;
  when ?exit3 true may ;
  when ?enterR1 true may ;
  when ?enterR2 true may ;
  when ?enterR3 true may ;
  when !lower exists i: i>2, (x[i]==CUTOFF) may goto down;
}
/*5*/
mode down true {
  when ?exit1 forall i:i>2 && i!=3, x[i]<=CUTOFF may goto c_must_raise;
  when ?exit2 forall i:i>2 && i!=4, x[i]<=CUTOFF may goto c_must_raise;
  when ?exit3 forall i:i>2 && i!=5, x[i]<=CUTOFF may goto c_must_raise;
  when ?exit1 exists i:i>2 && i!=3, CUTOFF<=x[i] may ;
  when ?exit2 exists i:i>2 && i!=4, CUTOFF<=x[i] may ;
  when ?exit3 exists i:i>2 && i!=5, CUTOFF<=x[i] may ;
  when ?enterR1 true may ;
  when ?enterR2 true may ;
  when ?enterR3 true may ;
}
/*6*/
urgent mode c_must_raise true {
  when !raise true may goto up;
}


/*7*/
mode far1 x <= 0 {
  rate x:1;
  when !enterR1 x==0 may goto R1;
}
/*8*/
mode R1 x <= 40 {
  rate x:1;
  when 30<=x && x <= 40 may goto I1;
}
/*9*/
mode I1 true {
  rate x:1;
  when !exit1 true may x in (-oo,0]; goto far1;
}

/*10*/
mode far2 x <= 0 {
  rate x:1;
  when !enterR2 x==0 may goto R2;
}
/*11*/
mode R2 x <= 40 {
  rate x:1;
  when 30<=x && x<=40 may goto I2;
}
/*12*/
mode I2 true {
  rate x:1;
  when !exit2 true may x in (-oo,0]; goto far2;
}

/*************************************************
*
	How this far3 can be deactivated ?
	1. invariance condition is the same as incoming assisgnment interval.
	2. no triggering condition with timing constant properly inside the incoming assignment interval.
	3. no specification timing constant properly inside the incoming assignment interval.
	4. no concurrent timing constant properly inside the incoming assignment interval.
	4. if there is a nonnegative rate, then the triggering timing bound happens at the upperside.
	5. if there is a nonpositive rate, then the triggering timing bound happens at the lowerside.
	then the far3 can be deactivated by
	a. eliminate the clock
	b. reinforce the invariance condition.
*/
/*13*/
mode far3 x <= 0 {
  rate x:1;
  when !enterR3 x==0 may goto R3;
}
/*14*/
mode R3 x <= 40 {
  rate x:1;
  when 30<=x && x<=40 may goto I3;
}
/*15*/
mode I3 true {
  rate x:1;
  when !exit3 true may x in (-oo,0]; goto far3;
}



initially
    far1[3] && x[3]<=0
&&  far2[4] && x[4]<=0
&&  far3[5] && x[5]<=0
&&  g_open[1] && x[1]==90
&&  up[2];

parametric risk I1[3] && CUTOFF<=40 && not g_closed[1];


