/*==============================================================================
  Paramater
  ============================================================================== */
process count = 7;

#define MinUuAckLag 5
#define MaxUuAckLag 10 
#define MinlubLag 5
#define MaxlubLag 10
#define SgnWasteTime 5
#define MinUuLag 5
#define MaxUuLag 10
#define MinUuDataLag 5
#define MaxUuDataLag 10


global  discrete Mmax;	/* Maximum number of preamble ramping cycle */
global 	discrete M:1..10; 	/* Counter, Message ReTransmite Counter */

global  dense  T2max;	/* Time Interval per message */

global  dense   Nb01min;	/* range of backoff interval for timer Tb01
				   given in terms of numbers of transmission
				   time intervals Nb01max and Nb01min, 
				   applicable when negative acknowledgement on AICH
				   is received.*/
global  dense   Nb01max, probP;


local	clock	x;		/* Timer, Backoff Time Counter */
local   clock  	y;		
 
global synchronizer		/* Event Trigger */
	CMAC_Config_REQ,
	CMAC_Config_REQ_CNF,
	PHY_Access_REQ,
	NoAck,
	NAck,
	Ack,	
	FindAvailCH,
	PHY_Access_Req,
	NoAvailCH,
	negativeCH,
	positiveCH,			
	SignatureCompare,
	RACH_DataEntry,	
	NodeB_RACH_DataEntry,
	negativeCH_NodeBUu,
	positiveCH_NodeBUu,
	MAC_Data_IND,		
	lub_MAC_Data_IND;	
/*==============================================================================
  User Equipment: RRC Layer (Layer 3)
  ============================================================================== */
	/* Syn OUT: CMAC-Config-REQ */



/* MAC-Config-REQ) */

mode RRC_ParaPreset x==0{
	when true may goto SetParaMmax;
}


mode 	SetParaMmax x==0{
	when true may Mmax = 1; goto SetParaT2max;
	when true may Mmax = 2; goto SetParaT2max;
	when true may Mmax = 3; goto SetParaT2max;
	when true may Mmax = 4; goto SetParaT2max;
	when true may Mmax = 5; goto SetParaT2max;
}

mode	SetParaT2max x==0 {
	when true may T2max = 1; goto SetParaNb01min;
	when true may T2max = 2; goto SetParaNb01min;
	when true may T2max = 3; goto SetParaNb01min;
	when true may T2max = 4; goto SetParaNb01min;
	when true may T2max = 5; goto SetParaNb01min;
	when true may T2max = 6; goto SetParaNb01min;
	when true may T2max = 7; goto SetParaNb01min;
	when true may T2max = 8; goto SetParaNb01min;
	when true may T2max = 9; goto SetParaNb01min;
	when true may T2max = 10; goto SetParaNb01min;
}

mode 	SetParaNb01min x==0 {
	when true may Nb01min = 1; goto SetParaNb01max;
	when true may Nb01min = 2; goto SetParaNb01max;
	when true may Nb01min = 3; goto SetParaNb01max;
	when true may Nb01min = 4; goto SetParaNb01max;
	when true may Nb01min = 5; goto SetParaNb01max;
	when true may Nb01min = 6; goto SetParaNb01max;
	when true may Nb01min = 7; goto SetParaNb01max;
}

mode 	SetParaNb01max x==0 {
	when 1>=Nb01min may Nb01max = 1; goto SetParaP;
	when 2>=Nb01min may Nb01max = 2; goto SetParaP;
	when 3>=Nb01min may Nb01max = 3; goto SetParaP;
	when 4>=Nb01min may Nb01max = 4; goto SetParaP;
	when 5>=Nb01min may Nb01max = 5; goto SetParaP;
	when 6>=Nb01min may Nb01max = 6; goto SetParaP;
	when 7>=Nb01min may Nb01max = 7; goto SetParaP;
	when 8>=Nb01min may Nb01max = 8; goto SetParaP;
}

mode 	SetParaP x==0 {
	when true may probP = 1; goto Config_Request;
	when true may probP = 2; goto Config_Request;
	when true may probP = 3; goto Config_Request;
	when true may probP = 4; goto Config_Request;
	when true may probP = 5; goto Config_Request;
	when true may probP = 6; goto Config_Request;
	when true may probP = 7; goto Config_Request;
	when true may probP = 8; goto Config_Request;
	when true may probP = 9; goto Config_Request;
	when true may probP = 10; goto Config_Request;
}

mode	Config_Request x==0 {
	when !CMAC_Config_REQ true may goto waitConfig_Response;
}

mode  	waitConfig_Response true {
	when ?CMAC_Config_REQ_CNF true may goto RRC_ParaPreset;
}

/*==============================================================================
  User Equipment: MAC Layer (Layer 2)
  ============================================================================== */
	/*============================================== 
		Syn OUT: PHY-Access-REQ						
	
	   	Syn IN:	CMAC-Config-REQ
	   		RACH_Error
	   		NoAck
	   		NAck
	   		Ack		
	   =============================================*/

/* Start RACH procedure
	Get RACH tx confrol parameters from RRC:P,Mmax,Nb01,....
	And Wait until any data will be transmitted
*/
mode StartMAClayer true {
	when ?CMAC_Config_REQ true may goto WaitMAC_Data_REQ;
}

mode WaitMAC_Data_REQ true {
	when true may goto StartRACH;			/* MAC_Data_REQ */
	when true may goto WaitMAC_Data_REQ;			/* No Data */
}

mode StartRACH true {
	when true may M=0; goto IncPreamTransCounter;
}

mode IncPreamTransCounter true {
	when true may M++1; goto ChkBoundParaM;
}


/* Collision Detection and prepare to Send PHY-Data-REQ*/

mode ChkBoundParaM true {
						/*Increase M, and Updata RACH tx control
						  parameters, and Set Timer T2
						  and goto CallPHY				*/
	when M <  Mmax may goto ChkSynCMAC_Config_REQ;
						/*If over transmit counter, goto Error
						  handling state				*/
	when M >= Mmax may goto RACHerror;
}

mode ChkSynCMAC_Config_REQ true {
	/* Sync to RRC */
	when !CMAC_Config_REQ_CNF true may goto waitRACHparaRefresh; 	/* Update RACH Tx control Parameters 	*/
	when true may goto SetTimer2;			/* Do nothing! 				*/
}

mode waitRACHparaRefresh true{
	/* Sync from RRC */
	when ?CMAC_Config_REQ true may goto SetTimer2;
}

mode SetTimer2 true {
	when true may x = 0; goto DrawRandomNumberR;
}

mode DrawRandomNumberR true{			/*Draw random number 0<=R<1*/
	when y==1 may goto FunctCD;
	when y==2 may goto FunctCD;
	when y==4 may goto FunctCD;
	when y==5 may goto FunctCD;
	when y==6 may goto FunctCD;
	when y==7 may goto FunctCD;
	when y==8 may goto FunctCD;
	when y==9 may goto FunctCD;
	when y==10 may goto FunctCD;
}

mode FunctCD y<=10 {
	when y <= probP may goto CallPHY_Access_REQ;
	when y > probP  may goto WaitExpiryT2_CD;
}


mode WaitExpiryT2_CD x <= T2max {
	when x >= T2max may goto ChkSynCMAC_Config_REQ;
}

mode CallPHY_Access_REQ true {
						/*Sent and Request to PHY layer		*/
	when !PHY_Access_REQ true may goto waitPHYstate;
}

/*	Waiting for PHY layer response
	Call PRACH Procedure.
*/
mode waitPHYstate true {
						/* No positive or negative acquisition
						   indicator corresponding to the selected
						   signature is detected in the downlink
						   access slot corresponding to the selected
						   uplink access slot
											*/
	when ?NoAck true may goto WaitExpiryT2_NoAck;
						/* negative acquisition occur
						   wait expiry timer T2
						   and set timer Tb01
											*/
	when ?NAck true may goto WaitExpiryT2_NAck;
						/* positive acquisition occur
						   wait expiry timer T2
											*/
	when ?Ack true may goto RACHend;
}

/* x->local clock --> T2 */

mode WaitExpiryT2_NoAck x <= T2max {
	when x >= T2max may goto ChkSynCMAC_Config_REQ;
}

mode WaitExpiryT2_NAck x <= T2max {
	when x >= T2max may x = 0; goto WaitExpiryTb01_NAck;
}

mode WaitExpiryTb01_NAck x<=Nb01max {
	when ((x >= Nb01min) and (x <= Nb01max)) may goto ChkSynCMAC_Config_REQ;
}

mode RACHend true {
	when true may goto StartMAClayer;
}

mode RACHerror true {
	when true may goto StartMAClayer;
}

/*==============================================================================
  User Equipment: PHYSICAL Layer (Layer 1)
  ============================================================================== */
	/*============================================== 
		Syn OUT: FindAvailCH					
			 NoAck
			 NAck
			 Ack
	
	   	Syn IN:	PHY-Access-Req
	   		NoAvailCH
	   		negativeCH
	   		positiveCH
	 		
	   =============================================*/
	   
mode waitCommand_UEPHY true {
						/* wait PHY-Data-Req from MAC layer*/										
	when ?PHY_Access_REQ true may goto ChkUsableCh;
}


mode ChkUsableCh true {
						/* No positive or negative acquisition
						   indicator corresponding to the selected
						   signature is detected in the downlink
						   access slot corresponding to the selected
						   uplink access slot
											*/
	when 	!FindAvailCH true may goto ChannelState;
						/* Negative Acquisition Indicator
											*/

}					

mode ChannelState true {
	/* Sync From UU */
	when	?NoAvailCH true may goto SentNoAck;	/* Commanded Preamble Power exceeds the maximum
							   allowed power by 6dB or Retransmission expiry */
	/* Sync From NodeB PHY */						   
	when 	?negativeCH true may goto SentNAck;	/* negative AI corresponding to the selected signature
							   is detected in the DL access slot corresponding to the
							   UL access slot.				*/
	when 	?positiveCH true may goto SentAck;
}

mode SentNoAck true {
	/* Sync To UE MAC */
	when 	!NoAck true may goto waitCommand_UEPHY;
}

mode SentNAck true {
	/* Sync To UE MAC */
	when  	!NAck	true may goto waitCommand_UEPHY;
}

mode SentAck true {
	/* Sync To UE MAC */
	when	!Ack	true may goto waitCommand_UEPHY;
}



/*==============================================================================
  Interface: Uu interface
  ============================================================================== */
/*	Receiver PHY-Access-CNF and decide network latence.	*/ 
	/*============================================== 
		Syn OUT: NoAvailCH 				
			 SignatureCompare
			 NodeB_RACH_DataEntry
	
	   	Syn IN:	FindAvailCH
	   		RACH_DataEntry	
	   		negativeCH_NodeBUu
	   		positiveCH_NodeBUu
	   =============================================*/

mode WaitSignal true {
	/* Sync From UE PHY */
	when ?FindAvailCH true may x=0; goto UEtoNodeB_Lag;	
	/* Sync From UE PHY */
	when ?RACH_DataEntry true may x=0; goto UEtoNodeB_DataLag;
	/* Sync From NodeB_PHY */
	when ?negativeCH_NodeBUu true may x=0; goto NodeBtoUE_NAckLag;
	/* Sync From NodeB_PHY */
	when ?positiveCH_NodeBUu true may x=0; goto NodeBtoUE_AckLag;
}

mode UEtoNodeB_DataLag x < MaxUuDataLag {
										/* Data transmission may be delayed between
										   MinUUDataLag ~ MaxUUDataLag		*/
	/* Sync to NodeB */
	when !NodeB_RACH_DataEntry (x >= MinUuDataLag and x < MaxUuDataLag) may goto WaitSignal;
}


mode UEtoNodeB_Lag x <= MaxUuLag {
										
	when (x >= MinUuLag and x < MaxUuLag) may goto NodeBFound;
										
	/* Sync to UE Phy */
	when !NoAvailCH (x >= MinUuLag and x <= MaxUuLag) may goto WaitSignal;
}

mode NodeBtoUE_NAckLag x <= MaxUuAckLag {
	/* Sync to UE PHY and delayed */
	when !negativeCH (x >= MinUuAckLag and x >= MaxUuAckLag) may goto WaitSignal;
}

mode NodeBtoUE_AckLag x <= MaxUuAckLag {
	/* Sync to UE PHY and delayed */
	when !positiveCH (x >= MinUuAckLag and x >= MaxUuAckLag) may goto WaitSignal;
}

mode NodeBFound true {
	/* Sync to NodeB PHY and Delayed*/							
	when !SignatureCompare true may goto WaitSignal;
}
	

/*==============================================================================
  NodeB: PHYSICAL Layer (Layer 1)
  ============================================================================== */
	/*============================================== 
		Syn OUT: negativeCH
			 positiveCH
			 MAC-Data-IND				
			 negativeCH_NodeBUu
			 positiveCH_NodeBUu
			
	   	Syn IN:	SignatureCompare
	   		NodeB_RACH_DataEntry
	   		
	   =============================================*/  
mode waitCommand_NodeBPHY true {
	/* Sync from Uu */
	when ?SignatureCompare true may x=0; goto PcsSignature;
	/* Sync from Uu */
									/* When Data arrive */
	when ?NodeB_RACH_DataEntry true may goto SentMAC_Data_IND;
}

mode PcsSignature x<= SgnWasteTime {
	/* Processing something and waste time*/
	when x == SgnWasteTime may goto ChkSignature;
}

mode ChkSignature x<=SgnWasteTime {
	/* Sync to Uu and Delayed */
	when !negativeCH_NodeBUu true may goto waitCommand_NodeBPHY;
	/* Sync to Uu and Delayed */
	when !positiveCH_NodeBUu true may goto waitCommand_NodeBPHY;
}

mode SentMAC_Data_IND true {
	/* Sync to lub */
									/* to hight layer */
	when !MAC_Data_IND true may goto waitCommand_NodeBPHY;
}

/*==============================================================================
  Interface: lub, 3G equivalent of GSM a-bis Interface
  ============================================================================== */
	/*============================================== 
		Syn OUT: lub_MAC_Data_IND
			
	   	Syn IN:	MAC_Data_IND
	=============================================*/    
  
mode fromNodeB true {
	/* Sync from NodeB */
	when ?MAC_Data_IND true may x=0; goto waitlubLag;
}

mode waitlubLag x <= MaxlubLag {
	when (x >= MinlubLag and x <= MaxlubLag) may goto sentToRNC;
}

mode sentToRNC true {
	/* Sync to RNC MAc */
	when !lub_MAC_Data_IND true may goto fromNodeB;
}


/*==============================================================================
  RNC: MAC Layer (Radio Network Controller)
       The Main Tasks of the RNC are to manage radio access bearers for user data
       transport, manage and optimize radio network resource use and control mobility.
  ============================================================================== */
  
mode waitCommand_RNC true {
	/* Sync from lub */
	when ?lub_MAC_Data_IND true may goto EvalMACHeader;
}

mode EvalMACHeader true {
	when true may goto waitCommand_RNC;
}

initially RRC_ParaPreset[1] and x[1] == 0 and
		StartMAClayer[2] and x[2] ==0 and 
		waitCommand_UEPHY[3] and x[3] == 0 and 
		WaitSignal[4] and x[4] ==0 and 
		waitCommand_NodeBPHY[5] and x[5] ==0 and 
		fromNodeB[6] and x[6] ==0 and 
		waitCommand_RNC[7] and x[7] == 0;

risk waitCommand_RNC[7] and RACHend[2];
