#define	TRUE		1
#define	FALSE		0

/**************************
 command for queue events
*/
#define VIDEO_READY	0
#define TIMER1_OUT1	1
#define TIMER1_OUT2	2
#define ACK1		3
#define TIMER2_OUT1	4
#define TIMER2_OUT2	5
#define ACK2		6

/*************************************
 Period of video drive
*/ 
#define	PERIOD		20 

/*******************************************
 Constants for server operations
*/
#define ONE_FRAME_TIME	5

/*****************************************
 Constants for timers
*/
#define	TIMER_LONG	40
#define TIMER_SHORT	30

/*****************************************
 Constants for channels
*/
#define	CHANNEL_DELAY_UB	20
#define	CHANNEL_DELAY_LB	10


/****************************

process description: 
	1: for queue, 
	2: for camdrive,
	3: for server 
	4: channel1 
	5: timer1
	6: client1
	7: channel2
	8: timer2
	9: client2
*/
process count = 6; 
global discrete 
	qlen: 0..2; 
 
global discrete 
	q0_event,  
	on1, connect1; 

local clock x; 
global synchronizer 
	picture, signal_event, dequeue, 
	start_timer1_out1, timer1_out1, start_timer1_out2, timer1_out2, stop_timer1,  
	send_frame1, receive_frame1, ack_frame1, receive_ack1; 


/*********************************************************
* video server 
*/ 
mode queue true { 
/* 1 */	when ?picture !signal_event qlen = 0 may qlen++1; q0_event:= VIDEO_READY; 
/* 1 */	when ?timer1_out1 !signal_event qlen = 0 may qlen++1; q0_event:= TIMER1_OUT1; 
/* 1 */	when ?timer1_out2 !signal_event qlen = 0 may qlen++1; q0_event:= TIMER1_OUT2; 
/* 1 */	when ?receive_ack1 !signal_event qlen = 0 may qlen++1; q0_event:= ACK1; 

	when ?picture !signal_event qlen = 1 may qlen++1; 
/*10*/	when ?timer1_out1 !signal_event qlen = 1 may qlen++1; 
	when ?timer1_out2 !signal_event qlen = 1 may qlen++1; 
	when ?receive_ack1 !signal_event qlen = 1 may qlen++1; 

	when ?dequeue qlen>0 and qlen <= 1 may qlen--1; q0_event:= 0;
} 

/**********************************************************
* video driver 
*/ 
mode cam_drive x <= PERIOD { 
	when !picture x = PERIOD  may x:= 0; /* 1 */
}


/*********************************************************
* video server 
*/ 
mode server_idle (qlen = 0 or x = 0) { 
/*15*/	when qlen>0 and q0_event=VIDEO_READY and x=0 
		may x:= 0; on1:= TRUE; goto server_frames; 

	when !start_timer1_out2 !send_frame1 
	qlen>0 and q0_event=TIMER1_OUT1 and connect1=TRUE and x=0 
		may x:= 0; goto server_frame1; 

	when !dequeue 
	qlen>0 and q0_event=TIMER1_OUT2 and connect1=TRUE and x=0 
		may connect1:= FALSE; 

	when !stop_timer1 !dequeue 
	qlen>0 and q0_event=ACK1 and connect1=TRUE and x=0
		may ; 

	when ?signal_event true may x:= 0; 
} 

mode server_frames x<=ONE_FRAME_TIME {
	when !start_timer1_out1 !send_frame1 
	(connect1=TRUE and on1=TRUE and x=ONE_FRAME_TIME) 
		may x:= 0; on1:= FALSE; goto server_frames; 

	when !dequeue 
	(connect1=FALSE or on1=FALSE) and x=0 
		may x:= 0; goto server_idle; 

	when ?signal_event true may ; 
}

mode server_frame1 x<=ONE_FRAME_TIME {
	when !start_timer1_out2 !send_frame1 !dequeue 
	x=ONE_FRAME_TIME  
		may x:= 0; goto server_idle; 

	when ?signal_event true may ; 
}




/***********************************************************
* timer1
*/
mode timer1_idle true { 
	when ?start_timer1_out1 true may x:= 0; goto timer1_wait1; 
	when ?start_timer1_out2 true may x:= 0; goto timer1_wait2; 
	when ?stop_timer1 true may ; 
}

mode timer1_wait1 x<=TIMER_LONG { 
	when !timer1_out1 x=TIMER_LONG may goto timer1_idle; 
	when ?start_timer1_out1 true may ; 
	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}

mode timer1_wait2 x<=TIMER_SHORT { 
	when !timer1_out2 x=TIMER_SHORT may goto timer1_idle; 
	when ?start_timer1_out1 true may ; 
	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}


/********************************
 channel1
*/
mode channel1_idle true {
	when ?send_frame1 true may x:= 0; goto channel1_frame; 
	when ?ack_frame1 true may x:= 0; goto channel1_ack; 
}

mode channel1_frame x <= CHANNEL_DELAY_UB {
	when !receive_frame1 x >= CHANNEL_DELAY_LB may goto channel1_idle; 
}

mode channel1_ack x <= CHANNEL_DELAY_UB {
	when !receive_ack1 x >= CHANNEL_DELAY_LB may goto channel1_idle; 
}


/********************************
 client1
*/ 
mode client1_idle true {
	when ?receive_frame1 true may x:= 0; goto client1_ack; 
	when true may goto client1_dead; 
}

mode client1_ack x<= 1 {
	when !ack_frame1 x<= 1 may goto client1_idle; 
}

mode client1_dead true {
	when ?receive_frame1 true may; 
}


initially 
    queue[1] and x[1] = 0 and qlen = 0 and q0_event=0 
and on1=FALSE and connect1=TRUE 
and cam_drive[2] and x[2] = 0  
and server_idle[3] and x[3] = 0
and timer1_idle[4] and x[4] = 0
and channel1_idle[5] and x[5] = 0
and client1_idle[6] and x[6] = 0
;
 

risk 
	qlen >= 1
;
