#define	TRUE		1
#define	FALSE		0

/**************************
 command for queue events
*/
#define VIDEO_READY	0
#define TIMER1_OUT1	1
#define TIMER1_OUT2	2
#define ACK1		3
#define TIMER2_OUT1	4
#define TIMER2_OUT2	5
#define ACK2		6

/*************************************
 Period of video drive
*/ 
#define	PERIOD		20 

/*******************************************
 Constants for server operations
*/
#define ONE_FRAME_TIME	5

/*****************************************
 Constants for timers
*/
#define	TIMER_LONG	40
#define TIMER_SHORT	30

/*****************************************
 Constants for channels
*/
#define	CHANNEL_DELAY_UB	20
#define	CHANNEL_DELAY_LB	10


/****************************

process description: 
	1: for queue, 
	2: for camdrive,
	3: for server 
	4: channel1 
	5: timer1
	6: client1
	7: channel2
	8: timer2
	9: client2
*/
process count = 5; 
global discrete 
	qlen: 0..2; 
 
global discrete 
	q0_event, 
	on1, connect1; 


local clock x; 
global synchronizer 
	picture, signal_event, dequeue, 
	start_timer1_out1, timer1_out1, start_timer1_out2, timer1_out2, stop_timer1,  
	send_frame1, ack_frame1; 


/*********************************************************
* video server 
*/ 
/*m0*/
mode queue true { 
/* 1 */	when ?picture !signal_event qlen = 0 may qlen++1; q0_event:= VIDEO_READY; 
/* 2 */	when ?timer1_out1 !signal_event qlen = 0 may qlen++1; q0_event:= TIMER1_OUT1; 
/* 3 */	when ?timer1_out2 !signal_event qlen = 0 may qlen++1; q0_event:= TIMER1_OUT2; 
/* 4 */	when ?ack_frame1 !signal_event qlen = 0 may qlen++1; q0_event:= ACK1; 

/*5*/	when ?picture !signal_event qlen = 1 may qlen++1; 
	when ?timer1_out1 !signal_event qlen = 1 may qlen++1; 
/*7*/	when ?timer1_out2 !signal_event qlen = 1 may qlen++1; 
	when ?ack_frame1 !signal_event qlen = 1 may qlen++1; 

/*9*/	when ?dequeue qlen>0 and qlen <= 1 may qlen--1; q0_event:= 0; 
} 

/**********************************************************
* video driver 
*/ 
/*m1*/
mode cam_drive x<=15 { 
/*10*/	when !picture x=15  may x:= 0; /* 1 */
}


/*********************************************************
* video server 
*/ 
/*m2*/
mode server_idle (qlen = 0 or x = 0) { 
	when !dequeue 
		qlen>0 and q0_event=VIDEO_READY and x=0 
		may on1:= TRUE; goto server_frames; 

	when !dequeue !start_timer1_out2 !send_frame1 
		qlen>0 and q0_event=TIMER1_OUT1 and connect1=TRUE and x=0 
		may goto server_frame1; 
	when !dequeue 
		qlen>0 and q0_event=TIMER1_OUT2 and connect1=TRUE and x=0 
		may connect1:= FALSE; 
/*14*/	when !stop_timer1 !dequeue 
		qlen>0 and q0_event=ACK1 and connect1=TRUE and x=0
		may ; 

/*15*/	when ?signal_event true may x:= 0; 
} 

/*m3*/
mode server_frames true {
/*16*/	when !start_timer1_out1 !send_frame1 
		(connect1=TRUE and on1=TRUE) 
		may x:= 0; on1:= FALSE; goto server_frames; 

	when (connect1=FALSE or on1=FALSE) 
		may x:= 0; goto server_idle; 

	when ?signal_event true may ; 
}

/*m4*/
mode server_frame1 true {
	when !start_timer1_out2 !send_frame1  
		true 
		may x:= 0; goto server_idle; 

/*20*/	when ?signal_event true may ; 
}




/***********************************************************
* timer1
*/
/*m5*/
mode timer1_idle true { 
	when ?start_timer1_out1 true may goto timer1_wait1; 
	when ?start_timer1_out2 true may goto timer1_wait2; 
/*23*/	when ?stop_timer1 true may ; 
}

/*m6*/
mode timer1_wait1 true { 
/*24*/	when !timer1_out1 true may goto timer1_idle; 
/*25*/	when ?start_timer1_out1 true may ; 
	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}

/*m7*/
mode timer1_wait2 true { 
/*28*/	when !timer1_out2 true may goto timer1_idle; 
/*29*/	when ?start_timer1_out1 true may ; 
/*30*/	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}


/********************************
 client1
*/ 
/*m8*/
mode client1_idle true {
	when ?send_frame1 true may goto client1_ack; 
/*33*/	when true may goto client1_dead; 
}

/*m9*/
mode client1_ack true {
/*34*/	when !ack_frame1 true may goto client1_idle; 
/*35*/	when ?send_frame1 true may; 
}

/*m10*/
mode client1_dead true {
	when ?send_frame1 true may; 
}


initially 
    queue[1] and x[1] = 0 and qlen = 0 and q0_event=0 
and on1=FALSE and connect1=TRUE 
and cam_drive[2] and x[2] = 0  
and server_idle[3] and x[3] = 0
and timer1_idle[4] and x[4] = 0
and client1_idle[5] and x[5] = 0
;
 

risk 
	qlen >= 2
;
