#define	TRUE		1
#define	FALSE		0

/**************************
 command for queue events
*/
#define VIDEO_READY	0
#define TIMER1_OUT1	1
#define TIMER1_OUT2	2
#define ACK1		3
#define TIMER2_OUT1	4
#define TIMER2_OUT2	5
#define ACK2		6

/*************************************
 Period of video drive
*/ 
#define	PERIOD		20 

/*******************************************
 Constants for server operations
*/
#define ONE_FRAME_TIME	5

/*****************************************
 Constants for timers
*/
#define	TIMER_LONG	40
#define TIMER_SHORT	30

/*****************************************
 Constants for channels
*/
#define	CHANNEL_DELAY	20


/****************************

process description: 
	1: for queue, 
	2: for camdrive,
	3: for server 
	4: channel1 
	5: timer1
	6: client1
	7: channel2
	8: timer2
	9: client2
*/
process count = 9; 
global discrete 
	qlen: 0..5; 
 
global discrete 
	q0_event, q1_event, q2_event, q3_event, 
	on1, connect1, 
	on2, connect2; 

local clock x; 
global synchronizer 
	picture, dequeue, 
	start_timer1_out1, timer1_out1, start_timer1_out2, timer1_out2, stop_timer1,  
	send_frame1, receive_frame1, ack_frame1, receive_ack1, 
	start_timer2_out1, timer2_out1, start_timer2_out2, timer2_out2, stop_timer2,  
	send_frame2, receive_frame2, ack_frame2, receive_ack2; 


/*********************************************************
* video server 
*/ 
mode queue true { 
/* 1 */	when ?picture qlen = 0 may qlen++1; q0_event:= VIDEO_READY; 
/* 1 */	when ?timer1_out1 qlen = 0 may qlen++1; q0_event:= TIMER1_OUT1; 
/* 1 */	when ?timer1_out2 qlen = 0 may qlen++1; q0_event:= TIMER1_OUT2; 
/* 1 */	when ?receive_ack1 qlen = 0 may qlen++1; q0_event:= ACK1; 
/* 5 */	when ?timer2_out1 qlen = 0 may qlen++1; q0_event:= TIMER2_OUT1; 
/* 1 */	when ?timer2_out2 qlen = 0 may qlen++1; q0_event:= TIMER2_OUT2; 
	when ?receive_ack2 qlen = 0 may qlen++1; q0_event:= ACK2; 

	when ?picture qlen = 1 may qlen++1; q1_event:= VIDEO_READY; 
	when ?timer1_out1 qlen = 1 may qlen++1; q1_event:= TIMER1_OUT1; 
/* 10*/	when ?timer1_out2 qlen = 1 may qlen++1; q1_event:= TIMER1_OUT2; 
	when ?receive_ack1 qlen = 1 may qlen++1; q1_event:= ACK1; 
	when ?timer2_out1 qlen = 1 may qlen++1; q1_event:= TIMER2_OUT1; 
	when ?timer2_out2 qlen = 1 may qlen++1; q1_event:= TIMER2_OUT2; 
	when ?receive_ack2 qlen = 1 may qlen++1; q1_event:= ACK2; 

/* 15*/	when ?picture qlen = 2 may qlen++1; q2_event:= VIDEO_READY; 
	when ?timer1_out1 qlen = 2 may qlen++1; q2_event:= TIMER1_OUT1; 
	when ?timer1_out2 qlen = 2 may qlen++1; q2_event:= TIMER1_OUT2; 
	when ?receive_ack1 qlen = 2 may qlen++1; q2_event:= ACK1; 
	when ?timer2_out1 qlen = 2 may qlen++1; q2_event:= TIMER2_OUT1; 
/* 20*/	when ?timer2_out2 qlen = 2 may qlen++1; q2_event:= TIMER2_OUT2; 
	when ?receive_ack2 qlen = 2 may qlen++1; q2_event:= ACK2; 

	when ?picture qlen = 3 may qlen++1; q3_event:= VIDEO_READY; 
	when ?timer1_out1 qlen = 3 may qlen++1; q3_event:= TIMER1_OUT1; 
	when ?timer1_out2 qlen = 3 may qlen++1; q3_event:= TIMER1_OUT2; 
/* 25*/	when ?receive_ack1 qlen = 3 may qlen++1; q3_event:= ACK1; 
	when ?timer2_out1 qlen = 3 may qlen++1; q3_event:= TIMER2_OUT1; 
	when ?timer2_out2 qlen = 3 may qlen++1; q3_event:= TIMER2_OUT2; 
	when ?receive_ack2 qlen = 3 may qlen++1; q3_event:= ACK2; 

	when ?dequeue qlen > 0 may qlen--1; q0_event:= q1_event; q1_event:= q2_event; q2_event:= q3_event; 
} 

/**********************************************************
* video driver 
*/ 
mode cam_drive x <= PERIOD { 
/* 30*/	when !picture x = PERIOD  may x:= 0; /* 1 */
}


/*********************************************************
* video server 
*/ 
mode server_idle x <= 1 { 
	when qlen>0 and q0_event=VIDEO_READY and x<1 
		may x:= 0; on1:= TRUE; on2:= TRUE; goto server_frames; 

	when !start_timer1_out2 !send_frame1 
		qlen>0 and q0_event=TIMER1_OUT1 and connect1=TRUE and x<1 
		may x:= 0; goto server_frame1; 
	when !dequeue q0_event=TIMER1_OUT2 and connect1=TRUE and x<1 
		may connect1:= FALSE; 
	when !stop_timer1 !dequeue qlen > 0 and q0_event=ACK1 and connect1=TRUE and x <1
		may ; 

	when !start_timer2_out2 !send_frame2 
		qlen>0 and q0_event=TIMER2_OUT1 and connect2=TRUE and x<1 
		may x:= 0; goto server_frame2; 
	when !dequeue q0_event=TIMER2_OUT2 and connect2=TRUE and x<1 
		may connect1:= FALSE; 
	when !stop_timer2 !dequeue qlen > 0 and q0_event=ACK1 and connect1=TRUE and x <1
		may ; 
} 

mode server_frames x<=ONE_FRAME_TIME {
	when !start_timer1_out1 !send_frame1 
	(connect1=TRUE and on1=TRUE and x=ONE_FRAME_TIME) 
		may x:= 0; on1:= FALSE; goto server_frames; 

	when !start_timer2_out1 !send_frame2
	(connect2=TRUE and on2=TRUE and x=ONE_FRAME_TIME) 
		may x:= 0; on2:= FALSE; goto server_frames; 

	when !dequeue (connect1=FALSE or on1=FALSE) and (connect2=FALSE or on2=FALSE) and x=0 
		may goto server_idle; 
}

mode server_frame1 x<=ONE_FRAME_TIME {
	when !start_timer1_out2 !send_frame1 !dequeue 
	x=ONE_FRAME_TIME  
		may x:= 0; goto server_idle; 
}

mode server_frame2 x<=ONE_FRAME_TIME {
	when !start_timer2_out2 !send_frame2 !dequeue 
	x=ONE_FRAME_TIME  
		may x:= 0; goto server_idle; 
}




/***********************************************************
* timer1
*/
mode timer1_idle true { 
	when ?start_timer1_out1 true may x:= 0; goto timer1_wait1; 
	when ?start_timer1_out2 true may x:= 0; goto timer1_wait2; 
	when ?stop_timer1 true may ; 
}

mode timer1_wait1 x<=TIMER_LONG { 
	when !timer1_out1 x=TIMER_LONG may goto timer1_idle; 
	when ?start_timer1_out1 true may ; 
	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}

mode timer1_wait2 x<=TIMER_SHORT { 
	when !timer1_out2 x=TIMER_SHORT may goto timer1_idle; 
	when ?start_timer1_out1 true may ; 
	when ?start_timer1_out2 true may ; 
	when ?stop_timer1 true may goto timer1_idle; 
}


/********************************
 channel1
*/
mode channel1_idle true {
	when ?send_frame1 true may x:= 0; goto channel1_frame; 
	when ?ack_frame1 true may x:= 0; goto channel1_ack; 
}

mode channel1_frame x <= CHANNEL_DELAY {
	when !receive_frame1 x < CHANNEL_DELAY may goto channel1_idle; 
}

mode channel1_ack x <= CHANNEL_DELAY {
	when !receive_frame1 x < CHANNEL_DELAY may goto channel1_idle; 
}


/********************************
 client1
*/ 
mode client1_idle true {
	when ?receive_frame1 true may x:= 0; goto client1_ack; 
	when true may goto client1_dead; 
}

mode client1_ack x<= 1 {
	when !ack_frame1 x<= 1 may goto client1_idle; 
}

mode client1_dead true {
	when ?receive_frame1 true may; 
}


/*******************************************************************************
* All operations for the 2nd client
*/
/***********************************************************
* timer2
*/
mode timer2_idle true { 
	when ?start_timer2_out1 true may x:= 0; goto timer2_wait1; 
	when ?start_timer2_out2 true may x:= 0; goto timer2_wait2; 
	when ?stop_timer2 true may ; 
}

mode timer2_wait1 x<=TIMER_LONG { 
	when !timer2_out1 x=TIMER_LONG may goto timer2_idle; 
	when ?start_timer2_out1 true may ; 
	when ?start_timer2_out2 true may ; 
	when ?stop_timer2 true may goto timer2_idle; 
}

mode timer2_wait2 x<=TIMER_SHORT { 
	when !timer2_out2 x=TIMER_SHORT may goto timer2_idle; 
	when ?start_timer2_out1 true may ; 
	when ?start_timer2_out2 true may ; 
	when ?stop_timer2 true may goto timer2_idle; 
}

/********************************
 channel2
*/
mode channel2_idle true {
	when ?send_frame2 true may x:= 0; goto channel2_frame; 
	when ?ack_frame2 true may x:= 0; goto channel2_ack; 
}

mode channel2_frame x <= CHANNEL_DELAY {
	when !receive_frame1 x < CHANNEL_DELAY may goto channel2_idle; 
}

mode channel2_ack x <= CHANNEL_DELAY {
	when !receive_frame2 x < CHANNEL_DELAY may goto channel2_idle; 
}


/********************************
 client2
*/ 
mode client2_idle true {
	when ?receive_frame2 true may x:= 0; goto client2_ack; 
	when true may goto client2_dead; 
}

mode client2_ack x<= 1 {
	when !ack_frame2 x<= 1 may goto client2_idle; 
}

mode client2_dead true {
	when ?receive_frame2 true may; 
}


initially 
    queue[1] and x[1] = 0 and qlen = 0 and on1=FALSE and on2 = FALSE and connect1=TRUE and connect1=TRUE 
and cam_drive[2] and x[2] = 0  
and server_idle[3] and x[3] = 0
and timer1_idle[4] and x[4] = 0
and channel1_idle[5] and x[5] = 0
and client1_idle[6] and x[6] = 0
and timer2_idle[7] and x[7] = 0
and channel2_idle[8] and x[8] = 0
and client2_idle[9] and x[9] = 0
;
 

risk 
	qlen >= 5
;
